@@ -4,6 +4,7 @@
 # This source code is licensed under the license found in the
 # LICENSE file in the root directory of this source tree.
 
+import functools
 import inspect
 from typing import Optional, List
 
@@ -35,3 +36,23 @@ def simple_repr(obj, attrs: Optional[List[str]] = None, overrides={}):
         if display:
             attrs_repr.append(f"{attr}={value}")
     return f"{obj.__class__.__name__}({','.join(attrs_repr)})"
+
+
+def capture_init(init):
+    """capture_init.
+
+    Decorate `__init__` with this, and you can then
+    recover the *args and **kwargs passed to it in `self._init_args_kwargs`
+    """
+    signature = inspect.signature(init)
+
+    @functools.wraps(init)
+    def __init__(self, *args, **kwargs):
+        bound = signature.bind(self, *args, **kwargs)
+        actual_kwargs = dict(bound.arguments)
+        del actual_kwargs['self']
+        actual_kwargs.update(bound.kwargs)
+        self._init_kwargs = actual_kwargs
+        init(self, *args, **kwargs)
+
+    return __init__
